﻿using ProtoBuf;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Utility
{
    [ProtoContract]
    public class Month
    {
        [ProtoMember(1)]
        DateTime innerDateTime = DateTime.MinValue;

        public Month(DateTime dateTime)
        {
            this.innerDateTime = dateTime.FirstDayOfMonth().Date;
        }

        public Month()
        {
        }

        public static Month MinValue
        {
            get
            {
                return new Month(DateTime.Now.AddYears(-10));
            }
        }

        public static Month MaxValue
        {
            get
            {
                return new Month(DateTime.Now.AddYears(10));
            }
        }

        /// <summary>
        /// 本月第一天
        /// </summary>
        public DateTime FirstDate
        {
            get
            {
                return innerDateTime.FirstDayOfMonth();
            }
        }

        /// <summary>
        /// 本月最后一天
        /// </summary>
        public DateTime LastDate
        {
            get
            {
                return innerDateTime.LastDayOfMonth();
            }
        }

        public DateTime InnerDateTime
        {
            get
            {
                return innerDateTime;
            }
        }

        public int ToInt()
        {
            return int.Parse(ToString());
        }

        /// <summary>
        /// 获得新日期
        /// </summary>
        /// <param name="date"></param>
        /// <returns></returns>
        public Date AddDays(int date)
        {
            DateTime dateTime = innerDateTime.AddDays(date);
            return new Date(dateTime);
        }

        public override bool Equals(object obj)
        {
            if (obj is Month)
            {
                return innerDateTime == ((Month)obj).innerDateTime;
            }
            return false;
        }

        public override int GetHashCode()
        {
            return ToString().GetHashCode();
        }

        public override string ToString()
        {
            return innerDateTime.ToYMDString();
        }

        public static bool operator ==(Month d1, Month d2)
        {
            bool d1_null = (d1 as object) == null;
            bool d2_null = (d2 as object) == null;

            if (d1_null && d2_null)
                return true;

            if ((d1_null && !d2_null) || (d2_null && !d1_null))
                return false;

            return d1.innerDateTime == d2.innerDateTime;
        }

        public static bool operator !=(Month d1, Month d2)
        {
            return !(d1 == d2);
        }

        public static bool operator <(Month t1, Month t2)
        {
            return t1.innerDateTime < t2.innerDateTime;
        }

        public static bool operator <=(Month t1, Month t2)
        {
            return t1.innerDateTime <= t2.innerDateTime;
        }

        public static bool operator >(Month t1, Month t2)
        {
            return t1.innerDateTime > t2.innerDateTime;
        }

        public static bool operator >=(Month t1, Month t2)
        {
            return t1.innerDateTime >= t2.innerDateTime;
        }
    }
}
